﻿/////////////////////////////////////////////////////////////////////////////////////////
//                              INTEL CONFIDENTIAL
//              Copyright 2018 Intel Corporation All Rights Reserved.
/////////////////////////////////////////////////////////////////////////////////////////
//
// The source code contained or described herein and all documents related to the source 
// code ("Material") are owned by Intel Corporation or its suppliers or licensors. Title 
// to the Material remains with Intel Corporation or its suppliers and licensors. The 
// Material may contain trade secrets and proprietary and confidential information of 
// Intel Corporation and its suppliers and licensors, and is protected by worldwide 
// copyright and trade secret laws and treaty provisions. No part of the Material may be 
// used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed, 
// or disclosed in any way without Intel’s prior express written permission. 
//
// No license under any patent, copyright, trade secret or other intellectual property 
// right is granted to or conferred upon you by disclosure or delivery of the Materials, 
// either expressly, by implication, inducement, estoppel or otherwise. Any license under
// such intellectual property rights must be express and approved by Intel in writing.
//
//////////////////////////////////////////////////////////////////////////

/************************************************************************************************************************
    Example for parsing metadata from a sample

    Guid myGuid = new Guid("F93AF74F-3525-4172-BF69-32AD6EF81D7A");
    AttributeExample myAttribute = Parse<AttributeExample>(frame.Properties, myGuid);

************************************************************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;

namespace AttributesUtils
{
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct AttributeExample
    {
        public UInt32 fieldA;
        public Byte fieldB;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2)]
        public Byte[] arrayC;
    };

    public class AttributeParser
    {
        public static T ByteArrayToStructure<T>(byte[] arr)
        {
            T str = default(T);
            int size = Marshal.SizeOf<T>();
            IntPtr ptr = Marshal.AllocHGlobal(size);
            Marshal.Copy(arr, 0, ptr, size);
            str = Marshal.PtrToStructure<T>(ptr);
            Marshal.FreeHGlobal(ptr);
            return str;
        }

        public static T Parse<T>(IReadOnlyDictionary<Guid, System.Object> Properties, Guid guid, out bool success)
        {
            try
            {
                object rawData;
                if (Properties.TryGetValue(guid, out rawData) == true)
                {
                    success = true;
                    return ByteArrayToStructure<T>((Byte[])rawData);
                }
            }
            catch (Exception ex)
            {

            }
            success = false;
            return default(T);
        }

        public static T Parse<T>(IReadOnlyDictionary<Guid, System.Object> Properties, Guid guid)
        {
            bool success = false;
            return Parse<T>(Properties, guid, out success);
        }
    }
}